# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.

from qirrunner._native import run, Output


class OutputHandler:
    """
    Callback handler for the output of the QIR runtime functions.
    Output is stored in a list and can be retrieved as a single string.
    """

    def __init__(self):
        self._output = []

    def handle(self, output: Output) -> None:
        """
        Append the output to the internal list.
        """
        self._output.append(str(output))

    def get_output(self) -> str:
        """
        Return the output as a single string.
        """
        return "".join(self._output)


__all__ = ["run", "Output", "OutputHandler"]



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.

from qirrunner import run, OutputHandler


def test_run_writes_to_callback() -> None:
    path = "./runner/tests/resources/bv.bc"

    handler = OutputHandler()
    run(path, shots=2, output_fn=handler.handle)
    expected = """START
METADATA\tEntryPoint
INFO\t[One, One, Zero]
END\t0
START
METADATA\tEntryPoint
INFO\t[One, One, Zero]
END\t0
"""
    assert expected == handler.get_output().replace("\r\n", "\n")



# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.

from typing import Callable, List, Optional

class Output:
    """
    An output recording from the QIR runtime functions.
    These are normally printed to the console.
    """

    def __repr__(self) -> str: ...
    def __str__(self) -> str: ...
    def _repr_html_(self) -> str: ...

def run(
    path: str,
    entry_point: Optional[str] = None,
    shots: Optional[int] = None,
    rng_seed: Optional[int] = None,
    output_fn: Optional[Callable[[Output], None]] = None,
):
    """
    Runs the supplied QIR (bitcode or textual IR) file.
    :param path: (Required) Path to the QIR file to run
    :param entry_point: Name of the entry point function to execute.
        Default is `None`.
    :param shots: The number of times to repeat the execution of
        the chosen entry point in the program. Default is 1.
    :param rngseed: The value to use when seeding the random number generator
        used for quantum simulation.
    :param output_fn: A callback function to receive the output of the QIR
        runtime functions. Default is `None`. When no callback is provided,
        the output is printed to the console.
    """

def main(args: Optional[List[str]]):
    """
    Main function for the QIR runner. This is only used when the runner is
    invoked from the command line and should not be used in other contexts.
    """
